import 'dart:async';
import 'dart:io';
import 'dart:ui';

import 'package:advertising_id/advertising_id.dart';
import 'package:device_info/device_info.dart';
import 'package:event_tracker/util/WtiteKeyUtils.dart';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter/services.dart';
import 'package:http/http.dart' as http;
import 'package:package_info_plus/package_info_plus.dart';
import 'package:platform_device_id_platform_interface/platform_device_id_platform_interface.dart';

/// Provides device id information.
class LemniskPlateform {
  /// Provides device and operating system information.
  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();

  /// Information derived from `android`-`androidId` or `ios`-`identifierForVendor`
  static Future<String?> get getDeviceId async {
    String? deviceId;
    try {
      if (kIsWeb) {
        deviceId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      } else if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await deviceInfoPlugin.androidInfo;
        // androidInfo.
        deviceId = androidInfo.androidId;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await deviceInfoPlugin.iosInfo;
        deviceId = iosInfo.identifierForVendor;
      } else {
        deviceId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceId = '';
    }
    return deviceId;
  }

  //device type
  static Future<String?> get getPlatFormDeviceType async {
    String? deviceType;
    try {
      if (Platform.isAndroid) {
        deviceType = "Android";
      } else if (Platform.isIOS) {
        deviceType = "IOS";
      }
    } on PlatformException {
      deviceType = '';
    }
    return deviceType;
  }

  //device model
  static Future<String?> get getDeviceModel async {
    String? deviceModel = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await deviceInfoPlugin.androidInfo;

        deviceModel = androidInfo.model;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await deviceInfoPlugin.iosInfo;
        deviceModel = iosInfo.model;
      } else {
        deviceModel = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceModel = '';
    }
    return deviceModel;
  }

  //get manufacture model
  static Future<String?> get getManufactureModel async {
    String? deviceManufacture = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await deviceInfoPlugin.androidInfo;

        deviceManufacture = androidInfo.manufacturer;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await deviceInfoPlugin.iosInfo;
        deviceManufacture = iosInfo.systemName;
      } else {
        deviceManufacture = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceManufacture = '';
    }
    return deviceManufacture;
  }

  //get osversion
  static Future<String?> get getOSVersion async {
    String? deviceID = '';
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo androidInfo = await deviceInfoPlugin.androidInfo;

        deviceID = androidInfo.version.release;
      } else if (Platform.isIOS) {
        IosDeviceInfo iosInfo = await deviceInfoPlugin.iosInfo;
        deviceID = iosInfo.systemVersion;
      } else {
        deviceID = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      deviceID = '';
    }
    return deviceID;
  }

  //get google advertising id
  static Future<String?> get getGoogleadvertisingID async {
    String? advertisingId;
    //bool? isLimitAdTrackingEnabled;
    try {
      if (Platform.isAndroid) {
        try {
          advertisingId = await AdvertisingId.id(true);
        } on PlatformException {
          advertisingId = 'Failed to get platform version.';
        }
      } else if (Platform.isIOS) {
        advertisingId = await AdvertisingId.id(true);
      } else {
        advertisingId = (await PlatformDeviceIdPlatform.instance.getDeviceId());
      }
    } on PlatformException {
      advertisingId = '';
    }
    return advertisingId;
  }

  static Future<bool?> get getGoogleadvertisingIDBoolValue async {
    bool? isLimitAdTrackingEnabled;
    try {
      if (Platform.isAndroid) {
        try {
          isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
        } on PlatformException {
          isLimitAdTrackingEnabled = false;
        }
      } else if (Platform.isIOS) {
        isLimitAdTrackingEnabled = await AdvertisingId.isLimitAdTrackingEnabled;
      }
    } on PlatformException {
      isLimitAdTrackingEnabled = false;
    }
    return isLimitAdTrackingEnabled;
  }

  //get app information
  static Future<String?> get getScreenWidth async {
    var physicalScreenSize = window.physicalSize;
    var physicalWidth = physicalScreenSize.width;
    return physicalWidth.toString();
  }

  static Future<String> get getScreenHeight async {
    //Size in physical pixels
    var physicalScreenSize = window.physicalSize;
    var physicalHeight = physicalScreenSize.height;

    return physicalHeight.toString();
  }

  //get app name
  static Future<String> get getAppName async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.appName;
  }

  //get app version
  static Future<String> get getAppVersion async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.version;
  }

  //get app build number
  static Future<String> get getBuildNumber async {
    PackageInfo _packageInfo = PackageInfo(
      appName: 'Unknown',
      packageName: 'Unknown',
      version: 'Unknown',
      buildNumber: 'Unknown',
      buildSignature: 'Unknown',
    );
    final info = await PackageInfo.fromPlatform();
    _packageInfo = info;
    return _packageInfo.buildNumber;
  }

  //get network ip
  static Future<String?> get getNetworkIP async {
    final url = Uri.parse('https://api.ipify.org/');
    final response = await http.get(url);
    // Future<Response> res=  response;
    // String s= (await res) as String;
    return response.body;
    //return "hbehbeebfe";
  }

  //write key static
  static Future<String> get getWriteKey async {
    try {
      if (Platform.isAndroid) {
        return WtiteKeyUtils.writekeyAndroid;
      } else if (Platform.isIOS) {
        return WtiteKeyUtils.writekeyIOS;
      }
    } on PlatformException {}
    return "";
  }

  // static Future<String> get getScreenDensity async {
  //   try {
  //     if (Platform.isAndroid) {
  //       return ScreenUtil().pixelRatio.toString();
  //     } else if (Platform.isIOS) {
  //       return ScreenUtil().pixelRatio.toString();
  //     }
  //   } on PlatformException {}
  //   return "";
  // }

  static Future<dynamic> get getReceivedAt async {
    return DateTime.now().toUtc().toString();
  }

  static Future<String?> get getSendAt async {
    return DateTime.now().toUtc().toString();
  }

  static Future<String?> get getTimestemp async {
    return DateTime.now().toUtc().toString();
  }

  static Future<String?> get getOriginalTimetemp async {
    return DateTime.now().toUtc().toString();
  }
}
